from scrapy import Request
from scrapy.exceptions import NotConfigured
from urllib.parse import urlencode

from webscrapingapi_scrapy_sdk.request import WebScrapingApiRequest

class WebScrapingApiMiddleware:
    webscrapingapi_api_url = 'https://api.webscrapingapi.com/v1'

    def __init__(self, api_key):
        self.api_key = api_key

    @classmethod
    def from_crawler(cls, crawler):
        api_key = crawler.settings.get('WEBSCRAPINGAPI_API_KEY')
        if not api_key:
            raise NotConfigured

        return cls(api_key=api_key)

    def _get_wsa_url(self, params):
        params.update({
            'api_key': self.api_key,
        })
        params = urlencode(params)
        return f'{self.webscrapingapi_api_url}?{params}'

    def process_request(self, request, spider):
        if not isinstance(request, WebScrapingApiRequest):
            return
        webscrapingapi_url = self._get_wsa_url(request.meta['webscrapingapi_params'])
        new_request = request.replace(cls=Request, url=webscrapingapi_url, meta=request.meta)
        return new_request


import copy
from scrapy import Request

class WebScrapingApiRequest(Request):
    def __init__(self, url: str, params: dict = None, data: dict = None, headers: dict = None, meta = None, **kwargs):
        meta = copy.deepcopy(meta) or {}
        
        if not params:
            params = {}

        params['url'] = url
        
        if headers:
            params['keep_headers'] = 1
        else:
            headers = {}

        meta['webscrapingapi_params'] = params;

        super().__init__(
            url,
            headers=headers,
            body=data,
            meta=meta,
            **kwargs
        )


import logging
import re

import scrapy


class HideApiKeyFormatter(logging.Formatter):
    def __init__(self, *args, **kwargs):
        super(HideApiKeyFormatter, self).__init__(*args, **kwargs)

    def format(self, record):
        formatted = super(HideApiKeyFormatter, self).format(record)
        return re.sub(r'api_key=[^&]*', 'api_key=HIDDEN', formatted)


class WebScrapingApiSpider(scrapy.Spider):
    def __init__(self, fmt, datefmt, *args, **kwargs):
        formatter = HideApiKeyFormatter(fmt=fmt, datefmt=datefmt)
        root = logging.getLogger()
        for h in root.handlers:
            h.setFormatter(formatter)
        super().__init__(*args, **kwargs)

    @classmethod
    def from_crawler(cls, crawler, *args, **kwargs):
        log_format = crawler.settings.get('LOG_FORMAT')
        log_dateformat = crawler.settings.get('LOG_DATEFORMAT')
        spider = cls(log_format, log_dateformat, *args, **kwargs)
        spider._set_crawler(crawler)
        return spider


__all__ = ['WebScrapingApiRequest', 'WebScrapingApiMiddleware', 'WebScrapingApiSpider']

from webscrapingapi_scrapy_sdk.request import WebScrapingApiRequest
from webscrapingapi_scrapy_sdk.middleware import WebScrapingApiMiddleware
from webscrapingapi_scrapy_sdk.spider import WebScrapingApiSpider